@@ -42,8 +42,9 @@
  #  knowledge of the CeCILL license and that you accept its terms.
  #
 */
+
 #ifndef gmic_version
-#define gmic_version 168
+#define gmic_version 210
 
 #include <cstdio>
 #include <cstring>
@@ -136,6 +137,12 @@ namespace cimg_library {
 
 #else // #ifndef gmic_build
 
+#ifdef __GNUC__
+#if defined(__FAST_MATH__)
+#error Using the -ffast-math CFLAG is known to lead to important issues in libgmic. Please disable it to compile.
+#endif
+#endif
+
 // Define private functions, used to compile libgmic.
 //---------------------------------------------------
 
@@ -159,15 +166,18 @@ static struct cimg_is_abort {
   bool value, *ptr;
   cimg_is_abort():value(false),ptr(&value) {}
 } _cimg_is_abort;
-#ifdef cimg_use_openmp
-#define cimg_test_abort() if (*_cimg_is_abort.ptr && !omp_get_thread_num()) throw CImgAbortException();
-#else
-#define cimg_test_abort() if (*_cimg_is_abort.ptr) throw CImgAbortException()
-#endif // #ifdef cimg_use_openmp
+#define cimg_abort_test() if (*_cimg_is_abort.ptr) throw CImgAbortException()
+
+inline double gmic_mp_ext(char *const str, void *const p_list);
+#define cimg_mp_ext_function(str) return ::gmic_mp_ext(str._data,&mp.listout)
+
 #endif // #ifdef cimg_use_abort
 #ifndef cimg_display
 #define cimg_display 0
 #endif // #ifndef cimg_display
+#ifndef cimg_appname
+#define cimg_appname "gmic"
+#endif // #ifndef cimg_appname
 #include "./CImg.h"
 
 #if cimg_OS==2
@@ -218,8 +228,15 @@ struct gmic {
             gmic_list<T> &images, gmic_list<char> &images_names,
             float *const p_progress=0, bool *const p_is_abort=0);
 
+  // These functions return (or init) G'MIC-specific paths.
+  static const char* path_user(const char *const custom_path=0);
+  static const char* path_rc(const char *const custom_path=0);
+  static bool init_rc(const char *const custom_path=0);
+
   // Functions below should be considered as *private*, and should not be
   // used in user's code.
+  template<typename T>
+  static bool search_sorted(const char *const str, const T& list, const unsigned int length, unsigned int &out_ind);
   static int _levenshtein(const char *const s, const char *const t,
                           gmic_image<int>& d, const int i, const int j);
   static int levenshtein(const char *const s, const char *const t);
@@ -229,10 +246,9 @@ struct gmic {
   static const char* basename(const char *const str);
   static char *strreplace_fw(char *const str);
   static char *strreplace_bw(char *const str);
-  static const char* path_user(const char *const custom_path=0);
-  static const char* path_rc(const char *const custom_path=0);
-  static bool init_rc(const char *const custom_path=0);
-  static const gmic_image<char>& uncompress_stdlib();
+  static unsigned int strescape(const char *const str, char *const res);
+  static const gmic_image<char>& decompress_stdlib();
+  static double mp_ext(char *const str, void *const p_list);
 
   template<typename T>
   void _gmic(const char *const commands_line,
@@ -240,9 +256,9 @@ struct gmic {
              const char *const custom_commands, const bool include_stdlib,
              float *const p_progress, bool *const p_is_abort);
 
-  inline const char * set_variable(const char *const name, const char *const value,
-                                   const char operation='=',
-                                   const unsigned int *const variables_sizes=0);
+  const char *set_variable(const char *const name, const char *const value,
+                           const char operation='=',
+                           const unsigned int *const variables_sizes=0);
 
   gmic& add_commands(const char *const data_commands, const char *const commands_file=0);
   gmic& add_commands(std::FILE *const file, const char *const filename=0);
@@ -254,9 +270,8 @@ struct gmic {
                                     const bool is_debug=false) const;
 
   gmic_image<unsigned int> selection2cimg(const char *const string, const unsigned int indice_max,
-                                          const gmic_list<char>& names,
-                                          const char *const command, const bool is_selection,
-                                          const bool allow_new_name, gmic_image<char>& new_name);
+                                          const gmic_list<char>& names, const char *const command,
+                                          const bool is_selection=true, gmic_image<char> *const new_name=0);
 
   gmic_image<char>& selection2string(const gmic_image<unsigned int>& selection,
                                      const gmic_list<char>& images_names,
@@ -267,7 +282,8 @@ struct gmic {
 
   template<typename T>
   void _gmic_substitute_args(const char *const argument, const char *const argument0,
-                             const char *const command, const gmic_list<T>& images);
+                             const char *const command, const char *const item,
+                             const gmic_list<T>& images);
 
   gmic& print(const char *format, ...);
   gmic& error(const char *format, ...);
@@ -278,7 +294,8 @@ struct gmic {
                                    gmic_list<T>& images, gmic_list<char>& images_names,
                                    gmic_list<T>& parent_images, gmic_list<char>& parent_images_names,
 				   const unsigned int *const variables_sizes,
-                                   const gmic_image<unsigned int> *const command_selection);
+                                   const gmic_image<unsigned int> *const command_selection,
+                                   const bool is_image_expr);
   template<typename T>
   gmic& print(const gmic_list<T>& list, const gmic_image<unsigned int> *const callstack_selection,
 	      const char *format, ...);
@@ -343,19 +360,20 @@ struct gmic {
              const gmic_image<unsigned int> *const command_selection);
 
   // Class variables.
+  static const char *native_commands_names[];
   static gmic_image<char> stdlib;
 
   gmic_list<char> *const commands, *const commands_names, *const commands_has_arguments,
     *const _variables, *const _variables_names, **const variables, **const variables_names,
     commands_files, callstack;
-  gmic_list<unsigned int> dowhiles, repeatdones;
+  gmic_image<unsigned int> dowhiles, fordones, repeatdones;
   gmic_image<unsigned char> light3d;
   gmic_image<char> status;
-  void *display_window;
+  void *display_windows;
 
   float focale3d, light3d_x, light3d_y, light3d_z, specular_lightness3d, specular_shininess3d, _progress, *progress;
   unsigned long reference_time;
-  unsigned int nb_carriages, debug_filename, debug_line, cimg_exception_mode;
+  unsigned int nb_dowhiles, nb_fordones, nb_repeatdones, nb_carriages, debug_filename, debug_line, cimg_exception_mode;
   int verbosity, render3d, renderd3d;
   bool is_released, is_debug, is_running, is_start, is_return, is_quit, is_double3d, is_debug_info, check_elif;
   const char *starting_commands_line;
@@ -391,6 +409,8 @@ struct gmic_exception {
   }
 };
 
+inline double gmic_mp_ext(char *const str, void *const p_list) { return gmic::mp_ext(str,p_list); }
+
 #endif // #ifndef gmic_version
 
 // Local Variables:
